/*
 * Copyright (c) 2016-2017, Parallels International GmbH
 *
 * This file is part of disp-helper. It's free software; you can redistribute
 * it and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 *
 * Our contact details: Parallels International GmbH, Vordergasse 59, 8200
 * Schaffhausen, Switzerland.
 */

#ifndef CLIENT_HPP_
#define CLIENT_HPP_

#include "Utils.hpp"

namespace Client
{

struct Balancer;
struct Collector
{
	Collector(ba::io_service& service_);

	void Run();

private:
	void Iteration();
	void PerfomanceIteration();
	void PrepareFilesystem() const;
	bool PrepareResults();
	void SendResults();
	bool ExecuteScripts(boost::function<bool (fs::directory_entry const&)> predicate_,
		boost::function<void ()> delay_) const;

private:
	ba::io_service& m_service;
	ba::deadline_timer m_timer;
	ba::deadline_timer m_perfomance_timer;

	typedef std::map<std::string,
		     boost::function<int (const char*, const char*, char**) > > senders_t;
	senders_t m_senders;
	fs::path const m_scriptsDir;
	fs::path const m_preliminaryDir;
	fs::path const m_finalResultsDir;
	fs::path const m_archive;
	fs::path const m_flagFile;
	boost::shared_ptr<Balancer> m_balancer;
};

} // namespace Client

#endif
