/*
 * Copyright (c) 2016-2017, Parallels International GmbH
 *
 * This file is part of disp-helper. It's free software; you can redistribute
 * it and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 *
 * Our contact details: Parallels International GmbH, Vordergasse 59, 8200
 * Schaffhausen, Switzerland.
 */

#include <curl/curl.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#define CAPATH "/usr/share/disp-helper/"
#define CACERT "/usr/share/disp-helper/ovz-cert.crt"
#define CRLFILE "/usr/share/disp-helper/ovz-crl.pem"

/* The minimum buffer size allowed to be set is 1024 (libcurl man page) */
#define CURL_LIMIT 1024

struct buffer
{
	char *data;
	size_t len;
};

size_t write_function(void *data_, size_t size_, size_t nmemb_, struct buffer *s_)
{

	/* Drop received data if we already have enough */
	if (s_->len >= CURL_LIMIT)
		return 0;
	size_t n = size_ * nmemb_;
	size_t new_len = s_->len + n;
	char *tmp = realloc(s_->data, new_len + 1);
	if (tmp == NULL)
		return 0;
	s_->data = tmp;
	memcpy(s_->data + s_->len, data_, n);
	s_->data[new_len] = '\0';
	s_->len = new_len;

	return n;
}

const char* protocol(void)
{
	return "CURL";
}

int sender(const char* file_, const char* server_, char** response_)
{
	CURL* hash = NULL;
	struct curl_httppost* post = NULL;
	struct curl_httppost* last = NULL;
	struct buffer s = { NULL, 0 };
	CURLcode r;

	hash = curl_easy_init();

	curl_easy_setopt(hash, CURLOPT_URL, server_);
	curl_formadd(&post, &last,
		CURLFORM_COPYNAME, "name",
		CURLFORM_FILE, file_,
		CURLFORM_END
	);
	curl_easy_setopt(hash, CURLOPT_HTTPPOST, post);
	curl_easy_setopt(hash, CURLOPT_BUFFERSIZE, CURL_LIMIT);
	curl_easy_setopt(hash, CURLOPT_WRITEFUNCTION, write_function);
	curl_easy_setopt(hash, CURLOPT_WRITEDATA, &s);
	curl_easy_setopt(hash, CURLOPT_SSL_VERIFYPEER, 1);
	curl_easy_setopt(hash, CURLOPT_SSL_VERIFYHOST, 2);
	if(access(CACERT, F_OK) != -1) {
		curl_easy_setopt(hash, CURLOPT_CAPATH, CAPATH);
	}
	if (access(CRLFILE, F_OK) == 0) {
		curl_easy_setopt(hash, CURLOPT_CRLFILE, CRLFILE);
	}
	/*curl_easy_setopt(hash, CURLOPT_VERBOSE, 1L);*/

	r = curl_easy_perform(hash);
	if(r != CURLE_OK) {
		free(s.data);
		return -r;
	}
	*response_ = s.data;
	curl_formfree(post);
	curl_easy_cleanup(hash);

	return 0;
}

